package com.g1.one_target_mobile_sdk

import androidx.annotation.NonNull
import com.g1.onetargetsdk.Analytics
import com.g1.onetargetsdk.Configuration
import com.google.gson.Gson
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** OneTargetMobileSdkPlugin */
class OneTargetMobileSdkPlugin : FlutterPlugin, MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel

//    private fun log(msg: String) {
//        Log.e("loitpp", msg)
//    }

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, Const.sdk)
        channel.setMethodCallHandler(this)
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            Const.getPlatformVersion -> {
                result.success("Android ${android.os.Build.VERSION.RELEASE}")
            }
            Const.setupTracking -> {
                setupTracking(call, result)
            }
            Const.trackEvent -> {
                trackEvent(call, result)
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    private fun setupTracking(@NonNull call: MethodCall, @NonNull result: Result) {
        val configuration = Configuration()
        val arguments = call.arguments as Map<*, *>?
//        log("setupTracking $arguments")
        arguments?.let { map ->
            val isEnvironmentDev = map["isEnvironmentDev"] as Boolean?
//            log("setupTracking isEnvironmentDev $isEnvironmentDev")
            if (isEnvironmentDev == null || isEnvironmentDev == true) {
                configuration.setEnvironmentDev()
            } else {
                configuration.setEnvironmentProd()
            }
            val writeKey = map["writeKey"] as String?
            configuration.writeKey = writeKey
//            log("setupTracking writeKey $writeKey")
            val isShowLog = map["isShowLog"] as Boolean?
            configuration.isShowLog = isShowLog ?: false
        }
        val isSetupSuccess = Analytics.setup(configuration)
        result.success(isSetupSuccess)
    }

    private fun trackEvent(@NonNull call: MethodCall, @NonNull result: Result) {
        val arguments = call.arguments as Map<*, *>?
//        log("trackEvent $arguments")
        if (arguments == null) {
//            log("null -> return")
            result.success(null)
            return
        }
        arguments.let { map ->
            val workSpaceId = map["workSpaceId"] as String?
            val identityId = map["identityId"] as HashMap<String, Any>?
            val eventName = map["eventName"] as String?
            val eventDate = map["eventDate"] as Long?
            val eventData = map["eventData"] as HashMap<String, Any>?

//            log("workSpaceId $workSpaceId")
//            log("identityId $identityId")
//            log("eventName $eventName")
//            log("eventDate $eventDate")
//            log("eventData $eventData")

            var mInput: Any? = null
            val gson = Gson()
            Analytics.trackEvent(
                workSpaceId = workSpaceId,
                identityId = identityId,
                eventName = eventName,
                eventDate = eventDate,
                eventData = eventData,
                onPreExecute = { input ->
                    mInput = input
//                    result.success(
//                        hashMapOf(
//                            "input" to Gson().toJson(input),
//                        )
//                    )
                },
                onResponse = { isSuccessful, code, response ->
                    result.success(
                        hashMapOf(
                            "input" to gson.toJson(mInput),
                            "isSuccessful" to isSuccessful,
                            "code" to code,
                            "response" to gson.toJson(response),
                        )
                    )
                },
                onFailure = { t ->
                    result.success(
                        hashMapOf(
                            "input" to gson.toJson(mInput),
                            "error" to t.message,
                        )
                    )
                }
            )
        }
    }
}
